# The driving force, SRH, band gap, and thermal voltage are arguments
# This expression modifies an SRH like expression
proc Schenk args {
  set name [lindex [info level 0] 0]
  lappend usage "$name Eg= ni= Nc= Nv= T= F=E Emfn=Emfn Emfp=Emfp Elec=Elec Hole=Hole Accept=Accept Donor=Donor mat=Silicon"
  lappend usage {The intrensic carrier concentration (ni) is in cm$^{-3}$.}
  lappend usage {The conduction band edge carrier concentration (Nc) is in cm$^{-3}$.}
  lappend usage {The valance band edge carrier concentration (Nv) is in cm$^{-3}$.}
  lappend usage {Temperature (T) is in Kelvin.}
  lappend usage {The driving force/field (F) is optional and defaults to E.}
  lappend usage {The magnitude of the hole quasi-Fermi gradient (Emfp) is optional and defaults to Emfp.}
  lappend usage {The magnitude of the electron quasi-Fermi gradient (Emfn) is optional and defaults to Emfn.}
  lappend usage {The donor concentration (Donor) is optional and defaults to Donor.}
  lappend usage {The acceptor concentration (Accept) is optional and defaults to Accept.}
  lappend usage {Hole concentration (Hole) is optional and defaults to Hole.}
  lappend usage {Electron concentration (Elec) is optional and defaults to Elec.}
  lappend usage {Material (mat) is optional and defaults to silicon.}
  set returns {Schnek band to band tunneling alagator string}
  set description {Very complicated band to band tunneling model that incorporates the effects of recombination on band to band tunneling.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set Eg [__GetValue $args Eg "$name Error: No band gap (Eg) is given"]
  set ni [__GetValue $args ni "$name Error: No intrensic carrier concentration (ni) given"]
  set Nc [__GetValue $args Nc "$name Error: No conduction band edge carrier concentration (Nc) given"]
  set Nv [__GetValue $args Nv "$name Error: No valance band edge carrier concentration (Nv) given"]
  set T [__GetValue $args T "$name Error: No temperature (T) specified"]
  set Accept [__GetValue $args Accept ]
  if {$Accept == ""} { set Accept Accept }
  set Donor [__GetValue $args Donor ]
  if {$Donor == ""} { set Donor Donor }
  set F [__GetValue $args F ]
  if {$F == ""} { set F E }
  set Emfn [__GetValue $args Emfn ]
  if {$Emfn == ""} { set Emfn Emfn }
  set Emfp [__GetValue $args Emfp ]
  if {$Emfp == ""} { set Emfp Emfp }
  set Hole [__GetValue $args Hole ]
  if {$Hole == ""} { set Hole Hole }
  set Elec [__GetValue $args Elec ]
  if {$Elec == ""} { set Elec Elec }
  set Material [__GetValue $args mat ]
  if {$Material == ""} { set Material Silicon }
  if {$Material != "Silicon"} { puts "$name Error: Materials other than silicon not currently supported"; exit; }

  set k 1.38066e-23
  set q 1.602e-19
  set Vt "([expr $k/$q]*$T)"

  # Density Correction
  set n ($Elec*(($ni/$Nc)^($Elec/($Elec+1e3)*$Emfn/$F)))
  set p ($Hole*(($ni/$Nv)^($Hole/($Hole+1e3)*$Emfp/$F)))
  solution add name= SchenkSRH const solve val= (($n*$p-$ni*$ni)/(($n+$ni)*($p+$ni)))
  # Model Proper
  set A 8.977e20 ;# V/(cm*eV^3/2)
  set B 2.14667e7 ;# 1/(cm*s*V^2)
  set hw 0.0186 ;# hbar omega eV
  set Fc_pm ($B*(SchenkSRH>0?($Eg+$hw):($Eg-$hw))^1.5)
  set Fc_mp ($B*(SchenkSRH>0?($Eg-$hw):($Eg+$hw))^1.5)
  # The minus sign is so we return a generation term
  return -(SchenkSRH*$A*$F^3.5*(($Fc_mp)^-1.5*exp(-$Fc_mp/$F)/(exp($hw/$Vt)-1)+($Fc_pm)^-1.5*exp(-$Fc_pm/$F)/(1-exp(-$hw/$Vt))))
}

