proc Vt args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T=?"
  lappend usage "The temperature T is in units of Kelvin, this can be a value or an alagator expression"
  set returns "Vt alagator string"
  set description {Computes the conduction band energy relative to the vacuum level. Affinity must be set to something reasonable }
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set T [__GetValue $args T "$name Error: No temperature (T) specified"]

  set k 1.38066e-23
  set q 1.602e-19
  return "([expr $k/$q]*$T)"
}
 
proc Econd args {
  set name [lindex [info level 0] 0]
  lappend usage "$name Eea=4.05 potential=DevPsi"
  lappend usage "The electron affinity (Eea) defaults to the value for silicon of 4.05"
  set returns "Conduction band alagator string"
  set description {Computes the conduction band energy relative to the vacuum level. Affinity must be set to something reasonable }
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set Eea [__GetValue $args Eea ]
  if {$Eea == ""} { set Eea 4.05 }
  set potential [__GetValue $args potential ]
  if {$potential == ""} { set potential DevPsi }

  return "(-($Eea)-($potential))"
}

proc Eval args {
  set name [lindex [info level 0] 0]
  lappend usage "$name Eg=Eg Eea=4.05 potential=DevPsi"
  lappend usage "The band gap parameter is in units of eV and must be set to account for any band gap narrowing effects"
  lappend usage "The electron affinity (Eea) defaults to the value for silicon of 4.05"
  lappend usage "The electrostatic potential (potential) defaults to the name DevPsi"
  set returns "Valance band alagator string"
  set description {Computes the valance band energy relative to the vacuum level with the band gap. Affinity must be set to something reasonable }
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set Eg [__GetValue $args Eg "$name Error: No band gap (Eg) specified"]
  set Eea [__GetValue $args Eea ]
  if {$Eea == ""} { set Eea 4.05 }
  set potential [__GetValue $args potential ]
  if {$potential == ""} { set potential DevPsi }

  #solution add name=Eval solve $Mat const val = "((-[pdbGetDouble $Mat Affinity])-([pdbGetDouble $Mat Eg])+(DevPsi))"
  return "(-($Eea)-($Eg)-($potential))"
}

proc Midgap args {
  set name [lindex [info level 0] 0]
  lappend usage "$name Eg=Eg mat=Silicon"
  lappend usage "Band gap (Eg) is an alagator expression band gap."
  lappend usage "Material (mat) is optional and defaults to silicon."
  set returns "Midgap alagator expression"
  set description {The midgap from conduction and valance band equations to use as an initial condition for the quasiFermi levels}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set Eg [__GetValue $args Eg "$name Error: No band gap (Eg) specified"]
  set Material [__GetValue $args mat ]
  if {$Material == ""} { set Material Silicon }

  return (([Econd potential=0 mat=$Material]+[Eval Eg=$Eg potential=0 mat=$Material])/2.)
}

proc CondFermiDirac args {
  set name [lindex [info level 0] 0]
  lappend usage "$name Ec=E Nc=v T=a quasiFermi=Qfn"
  lappend usage "The The band edge (Ec) parameter is in units of eV and is the energy of the band edge statistics are computed on."
  lappend usage "The number of carriers at the band edge is N and is in units of cm$^-3$."
  lappend usage "The temperature T is in units of Kelvin, this can be a value or an alagator expression"
  lappend usage "The quasi-Fermi level (quasiFermi) parameter is the expression needed to evaluate the quasi-Fermi of this band, default is Qfn."
  set returns "Carrier statistics alagator expression"
  set description {}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set Nc [__GetValue $args Nc "$name Error: No band edge carrier concentration (N) specified"]
  set Ec [__GetValue $args Ec "$name Error: No band edge (Ec) specified"]
  set T [__GetValue $args T "$name Error: No temperature (T) specified"]
  set quasiFermi [__GetValue $args quasiFermi]
  if {$quasiFermi == ""} { set quasiFermi Qfn }
  set k 1.38066e-23
  set q 1.602e-19
  set Vt "([expr $k/$q]*$T)"

  # only difference between conduction and valance band is
  #               ' this minus sign
  return "($Nc*F12(-(($Ec)+($quasiFermi))/($Vt)))"
}

proc ValFermiDirac args {
  set name [lindex [info level 0] 0]
  lappend usage "$name Ev=E Nv=v quasiFermi=Qfp T=a"
  lappend usage "The band edge (Ev) parameter is in units of eV and is the energy of the band edge statistics are computed on."
  lappend usage "The number of carriers at the band edge is N and is in units of cm$^-3$."
  lappend usage "The temperature T is in units of Kelvin, this can be a value or an alagator expression"
  lappend usage "The quasi-Fermi level (quasiFermi) parameter is the expression needed to evaluate the quasi-Fermi of this band, default is Qfp."
  set returns "Carrier statistics alagator expression"
  set description {}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set Nv [__GetValue $args Nv "$name Error: No band edge carrier concentration (N) specified"]
  set Ev [__GetValue $args Ev "$name Error: No band edge (Ev) specified"]
  set T [__GetValue $args T "$name Error: No temperature (T) specified"]
  set quasiFermi [__GetValue $args quasiFermi]
  if {$quasiFermi == ""} { set quasiFermi Qfp  }
  set k 1.38066e-23
  set q 1.602e-19
  set Vt "([expr $k/$q]*$T)"

  return "($Nv*F12((($Ev)+($quasiFermi))/($Vt)))"
}

proc IntrinsicConc args {
  set name [lindex [info level 0] 0]
  lappend usage "$name Eg=Eg Nc=Nc Nv=Nv T=a"
  lappend usage "The band gap (Eg) parameter is in units of eV and is the energy of the band edge statistics are computed on."
  lappend usage "The number of carriers at the band edge is N and is in units of cm$^-3$."
  lappend usage "The temperature T is in units of Kelvin, this can be a value or an alagator expression"
  lappend usage "The quasi-Fermi level (quasiFermi) parameter is the expression needed to evaluate the quasi-Fermi of this band, default is Qfn."
  set returns "Carrier statistics alagator expression"
  set description {Computes the intrinsic carrier concentration according to $\sqrt(N_cN_v)\exp(-\frac{Eg}{2V_t})$}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set Nc [__GetValue $args Nc "$name Error: No conduction band edge carrier concentration (Nc) specified"]
  set Nv [__GetValue $args Nv "$name Error: No conduction band edge carrier concentration (Nv) specified"]
  set Eg [__GetValue $args Eg "$name Error: No band gap (Eg) specified"]
  set T [__GetValue $args T "$name Error: No temperature (T) specified"]
  set k 1.38066e-23
  set q 1.602e-19
  set Vt "([expr $k/$q]*$T)"

  return "(sqrt($Nc*$Nv)*exp(-($Eg)/(2*$Vt)))"
}

proc CondBoltzman args {
  set name [lindex [info level 0] 0]
  lappend usage "$name Ec=E Nc=v T=a quasiFermi=Qfn"
  lappend usage "The The band edge (Ec) parameter is in units of eV and is the energy of the band edge statistics are computed on."
  lappend usage "The number of carriers at the band edge is N and is in units of cm$^-3$."
  lappend usage "The temperature T is in units of Kelvin, this can be a value or an alagator expression"
  lappend usage "The quasi-Fermi level (quasiFermi) parameter is the expression needed to evaluate the quasi-Fermi of this band, default is Qfn."
  set returns "Carrier statistics alagator expression"
  set description {}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set Nc [__GetValue $args Nc "$name Error: No band edge carrier concentration (N) specified"]
  set Ec [__GetValue $args Ec "$name Error: No band edge (Ec) specified"]
  set T [__GetValue $args T "$name Error: No temperature (T) specified"]
  set quasiFermi [__GetValue $args quasiFermi]
  if {$quasiFermi == ""} { set quasiFermi Qfn }
  set k 1.38066e-23
  set q 1.602e-19
  set Vt "([expr $k/$q]*$T)"

  # only difference between conduction and valance band is
  #                ' this minus sign
  return "($Nc*exp(-(($Ev)-($quasiFermi))/($Vt)))"
}

proc ValBoltzman args {
  set name [lindex [info level 0] 0]
  lappend usage "$name Ev=E Nv=v quasiFermi=Qfp T=a"
  lappend usage "The band edge (Ev) parameter is in units of eV and is the energy of the band edge statistics are computed on."
  lappend usage "The number of carriers at the band edge is N and is in units of cm$^-3$."
  lappend usage "The temperature T is in units of Kelvin, this can be a value or an alagator expression"
  lappend usage "The quasi-Fermi level (quasiFermi) parameter is the expression needed to evaluate the quasi-Fermi of this band, default is Qfp."
  set returns "Carrier statistics alagator expression"
  set description {}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set Nv [__GetValue $args Nv "$name Error: No band edge carrier concentration (N) specified"]
  set Ev [__GetValue $args Ev "$name Error: No band edge (Ev) specified"]
  set T [__GetValue $args T "$name Error: No temperature (T) specified"]
  set quasiFermi [__GetValue $args quasiFermi]
  if {$quasiFermi == ""} { set quasiFermi Qfp  }
  set k 1.38066e-23
  set q 1.602e-19
  set Vt "([expr $k/$q]*$T)"

  return "($Nv*exp((($Ev)-($quasiFermi))/($Vt)))"
}

