# The Crowell-Sze avalanche model based on Baraff's curves aprroximations [1].
# [1] Crowell, C. R., and S. M. Sze, “Temperature dependence of avalanche multiplication
#     in semiconductors,” Applied Physics Letters Vol. 9 (1966): 242–244
#
# The ionization coefficients has the following form:
#
# alpha_n = (1/lambda_n) * exp(C0(rn) + C1(rn)*xn + C2(rn)*xn*xn
# alpha_p = (1/lambda_p) * exp(C0(rp) + C1(rp)*xp + C2(rp)*xp*xp
#
# with
# C0(r) = -1.92 + 75.5*r - 757*r*r,
# C1(r) = 1.75e-2 - 11.9*r + 46*r*r,
# C2(r) = 3.9e-4 - 1.17*r + 11.5*r*r,
#
# lambda_n = lambda300_e * ( tanh(E_opt_ph/2KT) / tanh(E_opt_ph/2KT300) )
# lambda_p = lambda300_h * ( tanh(E_opt_ph/2KT) / tanh(E_opt_ph/2KT300) )
#
# rn = E_opt_ph/Ei_e
# rp = E_opt_ph/Ei_h
#
# where E_opt_ph is the optical phonon energy, lambda300_e, lambda300_h are the mean
# free paths for electrons and holes at 300K, Ei_e, Ei_h are the electron and hole
# ionization energies. The E_opt_ph, lambda300_e, lambda300_h, Ei_e, Ei_h are model
# parameters whose default values for Silicon are given in charon::Material_Properties.
#
# F is the driving field, which can be GradQuasiFermi, EParallelJ, EParallelJtot,
# EeffParallelJ, or EeffParallelJtot.
#
# For numerical reasons, the field F is computed as n*Fn/(n+n0) for electrons,
# and p*Fp/(p+p0) for holes. n0 and p0 are damping parameters and default to 0 and
# can be set in the Avalanche Generation section through eDrForceRefDens and
# hDrForceRefDens respectively. Using positive values for n0 and p0 can improve
# convergence for problems where strong generation–recombination occurs in regions
# with small density.

#proc CrowellSze { driveForce Qfn Qfp Elec Hole Emob Hmob }
proc __CrowellSzeHole { DrivingForce lattT Current } {
  
  set Fh $DrivingForce
  
  # Retrieve physical constants
  set eleQ    1.602176634e-19 ;#       electron charge in [C]
  set kbBoltz 8.61733034e-5   ;#       Boltzmann constant in [eV/K]
  
  # Compute 'lambda(T)' in the model [cm]
  set E_opt_ph    0.063
  set lambda300_h 3.8e-7
  
  proc TanH {x}	{
    set tanh "((exp($x)-exp(-$x))/(exp($x)+exp(-$x)))"
    return "$tanh"
  }
  
  set lambda_c "[ TanH "($E_opt_ph/(2.*$kbBoltz*$lattT))"] / [ TanH "($E_opt_ph/(2.*$kbBoltz*300.0))"]"
  set lambdah     "$lambda300_h * $lambda_c"
  
  # Compute absolute values of the current densities (scaled)
  #set normJh   "($Hmob*$Hole*sqrt(abs(dot($Qfp,$Qfp))))"
  #set normJtot "sqrt($Emob*$Elec*dot($Qfn,$Qfn))+$Hmob*$Hole*dot($Qfp,$Qfp)))"
  
  # Compute impact ionization coefficients [1/cm]
  set C00 -1.92  ; set C01 75.5 ; set C02 -757.0;
  set C10 1.75e-2; set C11 -11.9; set C12 46.0;
  set C20 3.9e-4 ; set C21 -1.17; set C22 11.5;
  
  set Ei_h     1.8
  set r        [ expr $E_opt_ph / $Ei_h ]
  set x        "($Ei_h /( $lambdah * ($Fh)))" ;# [1]
  set C0       [ expr $C00 + $C01*$r + $C02*$r*$r ]
  set C1       [ expr $C10 + $C11*$r + $C12*$r*$r ]
  set C2       [ expr $C20 + $C21*$r + $C22*$r*$r ]
  set iicoeffh "(exp($C0 + $C1*$x + $C2*$x^2) / $lambdah)"
  
  set normJh $Current
  set hAvaRate ($iicoeffh*$normJh/$eleQ)
  
  return $hAvaRate
  
}

proc __CrowellSzeElec { DrivingForce lattT Current } {
  
  set Fe $DrivingForce
  
  # Retrieve physical constants
  set eleQ    1.602176634e-19 ;#       electron charge in [C]
  set kbBoltz 8.61733034e-5   ;#       Boltzmann constant in [eV/K]
  
  # Compute 'lambda(T)' in the model [cm]
  set E_opt_ph    0.063
  set lambda300_e 6.2e-7
  
  proc TanH {x}	{
    set tanh "((exp($x)-exp(-$x))/(exp($x)+exp(-$x)))"
    return "$tanh"
  }
  
  set lambda_c    "[ TanH "($E_opt_ph/(2.*$kbBoltz*$lattT))"] / [ TanH "($E_opt_ph/(2.*$kbBoltz*300.0))"]"
  set lambdae     "$lambda300_e * $lambda_c"
  
  # Compute absolute values of the current densities (scaled)
  #set normJe   "($Emob*$Elec*sqrt(abs(dot($Qfn,$Qfn))))"
  #set normJtot "sqrt($Emob*$Elec*dot($Qfn,$Qfn))+$Hmob*$Hole*dot($Qfp,$Qfp)))"
  
  # Compute impact ionization coefficients [1/cm]
  set C00 -1.92  ; set C01 75.5 ; set C02 -757.0;
  set C10 1.75e-2; set C11 -11.9; set C12 46.0;
  set C20 3.9e-4 ; set C21 -1.17; set C22 11.5;
  
  set Ei_e     1.1
  set r        [ expr $E_opt_ph / $Ei_e ]
  set x        "($Ei_e /( $lambdae * ($Fe)))" ;# [1]
  set C0       [ expr $C00 + $C01*$r + $C02*$r*$r ]
  set C1       [ expr $C10 + $C11*$r + $C12*$r*$r ]
  set C2       [ expr $C20 + $C21*$r + $C22*$r*$r ]
  set iicoeffe "(exp($C0 + $C1*$x + $C2*$x^2) / $lambdae)"
  
  # Compute avalanche generation rate
  set normJe $Current
  set eAvaRate ($iicoeffe*$normJe/$eleQ)
  
  return $eAvaRate
}

# source DrivingForce.tcl
# # Get Qfn Equation
# puts [CrowellSzeHole [DrivingForceEField DevPsi Hole 1e18] 300 [Current Qfp Hmob Hole]]
# # Get Qfp Equation
# puts [CrowellSzeElec [DrivingForceEField DevPsi Elec 1e18] 300 [Current Qfn Emob Elec]]



