# /*
# The Okuto-Crowell avalanche model is by Y. Okuto and C. R. Crowell, "Threshold
# Energy Effect on Avalanche Breakdown Voltage in Semiconductor Junctions,"
# Solid-State Electronics, vol.18, no.2, pp.161-168, 1975.
#
# The ionization coefficient takes the following form:
#
# alpha(F) = a*[1+c*(T-Tref)] * F^gamma * exp[-(b*[1+d*(T-Tref)]/F)^delta],
#
# where a, b, c, d, gamma, and delta are model parameters whose default values are
# given in charon::Material_Properties. The default values are good in Silicon for
# the range of electric field from 1e5 V/cm to 1e6 V/cm. F is the driving field,
# i.e., GradQuasiFermi, EParallelJ, EParallelJtot, EeffParallelJ, or EeffParallelJtot.
# Eeff is computed by charon::FEM_ElectricField.
#
# For numerical reasons, the field F is computed as n*Fn/(n+n0) for electrons,
# and p*Fp/(p+p0) for holes. n0 and p0 are damping parameters and default to 0 and
# can be set in the Avalanche Generation section through eDrForceRefDens and hDrForceRefDens
# respectively.
# Using positive values for n0 and p0 can improve convergence for problems where
# strong generation–recombination occurs in regions with small density.
# */

proc __OkutoHole { DrivingForce lattT Current } {
  set Fh $DrivingForce
  # Holes
  set a_h     0.243   ;#a_h:[1/V]
  set b_h     6.53e5  ;#b_h:[V/cm]
  set c_h     5.35e-4 ;#c_h:[1/K]
  set d_h     5.67e-4 ;#d_h:[1/K]
  set gamma_h 1       ;#gamma_h:[1]
  set delta_h 2       ;#delta_h:[1]
  
  set exh      "exp(-($b_h*(1.+$d_h*($lattT-300.0))/$Fh) ^ ($delta_h))"
  set iicoeffh "($a_h * (1.+$c_h*($lattT-300.0)) * ($Fh ^ $gamma_h) * $exh)"
  
  set normJh $Current
  set eleQ    1.602176634e-19 ;#       electron charge in [C]
  set hAvaRate ($iicoeffh*$normJh/$eleQ)
}

proc __OkutoElec { DrivingForce lattT Current } {
  
  set Fe $DrivingForce
  #// Electrons
  set a_e     0.426   ;#a_e:[1/V]
  set b_e     4.81e5  ;#b_e:[V/cm]
  set c_e     3.05e-4 ;#c_e:[1/K]
  set d_e     6.86e-4 ;#d_e:[1/K]
  set gamma_e 1       ;#gamma_e:[1]
  set delta_e 2       ;#delta_e:[1]
  
  set exe      "exp(-($b_e*(1.+$d_e*($lattT-300.0))/$Fe) ^ ($delta_e))"
  set iicoeffe "($a_e * (1.+$c_e*($lattT-300.0)) * ($Fe ^ $gamma_e) * $exe)"
  
  set normJe $Current
  set eleQ    1.602176634e-19 ;#       electron charge in [C]
  set eAvaRate ($iicoeffe*$normJe/$eleQ)
  
  return $eAvaRate
}

# ## Get Qfn Equation
# #puts [lindex [Okuto Qfn Qfp Elec Hole Emob Hmob 300 1e15 1e15] 0]
# ## Get Qfp Equation
# #puts [lindex [Okuto Qfn Qfp Elec Hole Emob Hmob 300 1e15 1e15] 1]
# source DrivingForce.tcl
# # Get Qfn Equation
# puts [OkutoHole [DrivingForceEField DevPsi Hole 1e18] 300 [Current Qfp Hmob Hole]]
# # Get Qfp Equation
# puts [OkutoElec [DrivingForceEField DevPsi Elec 1e18] 300 [Current Qfn Emob Elec]]


