# /*
# The ionization coefficient takes the following form:
#
# alpha(F,T) = a(T) * exp(-(Ecrit(T)/F)^delta),
#
# where a(T) = a0+a1*T+a2*T^2, Ecrit(T) = E0 or Eg(T)/q/lamda(T),
# and lamda(T) = lamda300*tanh(hbarOmega/(2*kB*T)).
# The a0, a1, a2, delta, E0, lambda300, and hbarOmega are model parameters whose
# default values are given in charon::Material_Properties.
# F is the driving field, which can be GradQuasiFermi, EParallelJ, EParallelJtot,
# EeffParallelJ, or EeffParallelJtot. Eeff is computed by charon::FEM_ElectricField.
#
# For numerical reasons, the field F is computed as n*Fn/(n+n0) for electrons,
# and p*Fp/(p+p0) for holes. n0 and p0 are damping parameters and default to 0 and
# can be set in the Avalanche Generation section through eDrForceRefDens and
# hDrForceRefDens respectively. Using positive values for n0 and p0 can improve
# convergence for problems where strong generation–recombination occurs in regions
# with small density.
# */

proc __SelberherrHole { DrivingForce lattT Current } {
  
  set Fh $DrivingForce
  
  # Retrieve physical constants
  set eleQ    1.602176634e-19 ;#       electron charge in [C]
  set kbBoltz 8.61733034e-5   ;#       Boltzmann constant in [eV/K]
  
  # Holes
  set a0_h        1.528e6    ;#a0_h:[1/cm] = P_IONIZATION in  1.0         );
  set a1_h        0.0        ;#a1_h:[1/(cm.K)] = P_ION_1 in  1.0          );
  set a2_h        0.0        ;#a2_h:[1/(cm.K^2)] = P_ION_2 in  1.0        );
  set delta_h     1.0        ;#delta_h:[1] = EXP_II in  1.0               );
  set E0_h        2.036e6    ;#E0_h:[V/cm] = ECP.II in Selberherr, not in  1.0);
  set lambda300_h 6.32079e-7 ;#lambda300_h:[cm] = LAP300 in  1.0          );
  set hbarOmega_h 0.063      ;#hbarOmega_h:[eV] = OP_PH_EN in  1.0        );
  
  #// Compute 'a(T)' in the model [1/cm]
  set ah "$a0_h + $a1_h*$lattT + $a2_h*$lattT*$lattT" ;
  #// Compute 'lambda(T)' in the model [cm]
  set lambdah  "$lambda300_h * tanh($hbarOmega_h / (2.*$kbBoltz*$lattT))" ;
  set iicoeffh "$ah * exp(-($E0_h/$Fh) ^ ($delta_h))";
  
  set normJh $Current
  set hAvaRate ($iicoeffh*$normJh/$eleQ)
  
  return $hAvaRate
  
}

proc __SelberherrElec { DrivingForce lattT Current } {
  
  set Fe $DrivingForce
  
  # Retrieve physical constants
  set eleQ    1.602176634e-19 ;#       electron charge in [C]
  set kbBoltz 8.61733034e-5   ;#       Boltzmann constant in [eV/K]
  
  # Electrons
  set a0_e        7.03e5     ;#a0_e:[1/cm] = N_IONIZATION in  1.0         );
  set a1_e        0.0        ;#a1_e:[1/(cm.K)] = N_ION_1 in  1.0          );
  set a2_e        0.0        ;#a2_e:[1/(cm.K^2)] = N_ION_2 in  1.0        );
  set delta_e     1.0        ;#delta_e:[1] = EXN_II in  1.0               );
  set E0_e        1.231e6    ;#E0_e:[V/cm] = ECN.II in Selberherr, not in  1.0);
  set lambda300_e 10.4542e-7 ;#lambda300_e:[cm] = LAN300 in  1.0          );
  set hbarOmega_e 0.063      ;#hbarOmega_e:[eV] = OP_PH_EN in  1.0        );
  
  #// Compute 'a(T)' in the model [1/cm]
  set ae "$a0_e + $a1_e*$lattT + $a2_e*$lattT*$lattT" ;
  #// Compute 'lambda(T)' in the model [cm]
  set lambdae "$lambda300_e * tanh($hbarOmega_e / (2.*$kbBoltz*$lattT))" ;
  set iicoeffe "$ae * exp(-($E0_e/$Fe) ^ ($delta_e))";
  
  # Compute avalanche generation rate
  set normJe $Current
  set eAvaRate ($iicoeffe*$normJe/$eleQ)
  
  return $eAvaRate
  
}

# source DrivingForce.tcl
# # Get Qfn Equation
# puts [SelberherrHole [DrivingForceEField DevPsi Hole 1e18] 300 [Current Qfp Hmob Hole]]
# # Get Qfp Equation
# puts [SelberherrElec [DrivingForceEField DevPsi Elec 1e18] 300 [Current Qfn Emob Elec]]


