# The New University of Bologna impact ionization model (UniBoNew) is a further
# developed version of the UniBo model by [1][2][3] to cover an extended temperature
# range between 25 celcius degree and 500 celcius degree (773 K). It is based on
# impact ionization data generated by the Boltzmann solver HARM [4] and is calibrated
# against specially designed impact ionization measurements [5][6]. It covers
# a wide range of electric fields.
#
# [1] E. Gnani et al., “Extraction method for the impact-ionization multiplication factor in
# silicon at large operating temperatures,” in Proceedings of the 32nd European Solid-
# State Device Research Conference (ESSDERC), Florence, Italy, pp. 227–230,
# September 2002.
# [2] S. Reggiani et al., “Investigation about the High-Temperature Impact-Ionization
# Coefficient in Silicon,” in Proceedings of the 34th European Solid-State Device
# Research Conference (ESSDERC), Leuven, Belgium, pp. 245–248, September 2004.
# [3] S. Reggiani et al., “Experimental extraction of the electron impact-ionization coefficient
# at large operating temperatures,” in IEDM Technical Digest, San Francisco, CA, USA,
# pp. 407–410, December 2004.
# [4] M. C. Vecchi and M. Rudan, “Modeling Electron and Hole Transport with Full-Band
# Structure Effects by Means of the Spherical-Harmonics Expansion of the BTE,” IEEE
# Transactions on Electron Devices, vol. 45, no. 1, pp. 230–238, 1998.
# [5] S. Reggiani et al., “Electron and Hole Mobility in Silicon at Large Operating
# Temperatures—Part I: Bulk Mobility,” IEEE Transactions on Electron Devices, vol. 49,
# no. 3, pp. 490–499, 2002.
# [6] M. Valdinoci et al., “Impact-ionization in silicon at large operating temperature,” in
# International Conference on Simulation of Semiconductor Processes and Devices
# (SISPAD), Kyoto, Japan, pp. 27–30, September 1999.
#
# The ionization coefficient takes the following form:
#
# α(F, T) = F / [a(T) + b(T) * exp( d(T)/(F + c(T)) ) ], where
#
# a(T)=sum_k(a_k*(T/1K)^k) (k=0 to 3), b(T)=sum_k(b_k*(T/1K)^k) (k=0 to 10),
# c(T)=sum_k(c_k*(T/1K)^k) (k=0 to 3), d(T)=sum_k(d_k*(T/1K)^k) (k=0 to 3).
#
# a_k, b_k, c_k, d_k are model parameters whose default values are given in
# charon::Material_Properties. F is the driving field, i.e., GradQuasiFermi,
# EParallelJ, EParallelJtot, EeffParallelJ, or EeffParallelJtot.
# Eeff is computed by charon::FEM_ElectricField.
#
# For numerical reasons, the field F is computed as n*Fn/(n+n0) for electrons,
# and p*Fp/(p+p0) for holes. n0 and p0 are damping parameters and default to 0 and
# can be set in the Avalanche Generation section through eDrForceRefDens and
# hDrForceRefDens respectively. Using positive values for n0 and p0 can improve
# convergence for problems where strong generation–recombination occurs in regions
# with small density.

proc __UniBoNewHole { DrivingForce lattT Current } {
  
  set Fh $DrivingForce
  
  # Retrieve physical constants
  set eleQ    1.602176634e-19 ;#       electron charge in [C]
  
  # Holes
  set a0_h  2.26018      ;#a0_h:[V]
  set a1_h  0.0134001    ;#a1_h:[V]
  set a2_h  -5.87724e-6  ;#a2_h:[V]
  set a3_h  -1.14021e-9  ;#a3_h:[V]
  set b0_h  0.058547     ;#b0_h:[V]
  set b1_h  -1.95755e-4  ;#b1_h:[V]
  set b2_h  2.44357e-7   ;#b2_h:[V]
  set b3_h  -1.33202e-10 ;#b3_h:[V]
  set b4_h  2.68082e-14  ;#b4_h:[V]
  set b5_h  0.0          ;#b5_h:[V]
  set b6_h  0.0          ;#b6_h:[V]
  set b7_h  0.0          ;#b7_h:[V]
  set b8_h  0.0          ;#b8_h:[V]
  set b9_h  0.0          ;#b9_h:[V]
  set b10_h 0.0          ;#b10_h:[V]
  set c0_h  1.95399e4    ;#c0_h:[V/cm]
  set c1_h  -104.441     ;#c1_h:[V/cm]
  set c2_h  0.498768     ;#c2_h:[V/cm]
  set c3_h  0.0          ;#c3_h:[V/cm]
  set d0_h  2.07712e6    ;#d0_h:[V/cm]
  set d1_h  993.153      ;#d1_h:[V/cm]
  set d2_h  7.77769      ;#d2_h:[V/cm]
  set d3_h  0.0          ;#d3_h:[V/cm]
  
  set normT [expr $lattT/1.0]
  
  set ah "($a0_h + $a1_h*$normT + $a2_h*pow($normT,2.) + $a3_h*pow($normT,3.))"
	set bh "($b0_h + $b1_h*$normT + $b2_h*pow($normT,2.) + $b3_h*pow($normT,3.) \
		   + $b4_h*pow($normT,4.) + $b5_h*pow($normT,5.) + $b6_h*pow($normT,6.) \
		   + $b7_h*pow($normT,7.) + $b8_h*pow($normT,8.) \
      + $b9_h*pow($normT,9.)+ $b10_h*pow($normT,10.))"
  set ch "($c0_h + $c1_h*$normT + $c2_h*pow($normT,2.) + $c3_h*pow($normT,3.))"
  set dh "($d0_h + $d1_h*$normT + $d2_h*pow($normT,2.) + $d3_h*pow($normT,3.))"
  
  set iicoeffh "($Fh / ([expr $ah] + [expr $bh] * exp([expr $dh]/($Fh + [expr $ch])) ) )"
  
  set normJh $Current
  set hAvaRate ($iicoeffh*$normJh/$eleQ)
  
  return $hAvaRate
  
}

proc __UniBoNewElec { DrivingForce lattT Current } {
  
  set Fe $DrivingForce
  
  # Retrieve physical constants
  set eleQ    1.602176634e-19 ;#       electron charge in [C]
  
  # Electrons
  set a0_e  4.65403      ;#a0_e:[V]
  set a1_e  -8.76031e-3  ;#a1_e:[V]
  set a2_e  1.34037e-5   ;#a2_e:[V]
  set a3_e  -2.75108e-9  ;#a3_e:[V]
  set b0_e  -0.128302    ;#b0_e:[V]
  set b1_e  4.45552e-3   ;#b1_e:[V]
  set b2_e  -1.0866e-5   ;#b2_e:[V]
  set b3_e  9.23119e-9   ;#b3_e:[V]
  set b4_e  -1.82482e-12 ;#b4_e:[V]
  set b5_e  -4.82689e-15 ;#b5_e:[V]
  set b6_e  1.09402e-17  ;#b6_e:[V]
  set b7_e  -1.24961e-20 ;#b7_e:[V]
  set b8_e  7.55584e-24  ;#b8_e:[V]
  set b9_e  -2.28615e-27 ;#b9_e:[V]
  set b10_e 2.73344e-31  ;#b10_e:[V]
  set c0_e  7.76221e3    ;#c0_e:[V/cm]
  set c1_e  25.18888     ;#c1_e:[V/cm]
  set c2_e  -1.37417e-3  ;#c2_e:[V/cm]
  set c3_e  1.59525e-4   ;#c3_e:[V/cm]
  set d0_e  7.10481e5    ;#d0_e:[V/cm]
  set d1_e  3.98594e3    ;#d1_e:[V/cm]
  set d2_e  -7.19956     ;#d2_e:[V/cm]
  set d3_e  6.96431e-3   ;#d3_e:[V/cm]
  
  set normT [expr $lattT/1.0]
  set ae "($a0_e + $a1_e*$normT + $a2_e*pow($normT,2.) + $a3_e*pow($normT,3.))"
	set be "($b0_e + $b1_e*$normT + $b2_e*pow($normT,2.) + $b3_e*pow($normT,3.)\
	       + $b4_e*pow($normT,4.) + $b5_e*pow($normT,5.)+$b6_e*pow($normT,6.)\
		   + $b7_e*pow($normT,7.) + $b8_e*pow($normT,8.) \
      + $b9_e*pow($normT,9.) + $b10_e*pow($normT,10.))"
  set ce "($c0_e + $c1_e*$normT + $c2_e*pow($normT,2.) + $c3_e*pow($normT,3.))"
  set de "($d0_e + $d1_e*$normT + $d2_e*pow($normT,2.) + $d3_e*pow($normT,3.))"
  
  set iicoeffe "($Fe / ([expr $ae] + [expr $be] * exp([expr $de]/($Fe + [expr $ce])) ) )"
  
  # Compute avalanche generation rate
  set normJe $Current
  set eAvaRate ($iicoeffe*$normJe/$eleQ)
  
  return $eAvaRate
  
}

# source DrivingForce.tcl
# # Get Qfn Equation
# puts [UniBoNewHole [DrivingForceEField DevPsi Hole 1e18] 300 [Current Qfp Hmob Hole]]
# # Get Qfp Equation
# puts [UniBoNewElec [DrivingForceEField DevPsi Elec 1e18] 300 [Current Qfn Emob Elec]]

