namespace eval Silicon {
# Philips Unified Mobility Model
# Daniel Cummings
# Last Updated 11/30/2006

# INPUTS Required:
# 1. ND - Donor Doping
# 2. NA - Acceptor Doping
# 3. Elec -  Electron Density
# 4. Hole -  Hole Density

# OUTPUT Terms:
# 1. Emob_Philips - Electron Mobility
# 2. Hmob_Philips - Hole Mobility

# Notes:
# ND and NA must be at least 1.0 for all regions

#======================================================
# 		       PROCEDURE
#======================================================

proc Cummings args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T= Accept=Accept Donor=Donor Elec=Elec Hole=Hole expand"
  lappend usage {Temperature (T) is in Kelvin.}
  lappend usage {Donor concentration (Donor) is assumed to be the alagator field Donor and dopant type As.}
  lappend usage {Accept concentration (Accept) is assumed to be the alagator field Accept.}
  lappend usage {Hole concentration (Hole) is assumed to be the alagator field Hole.}
  lappend usage {Elec concentration (Elec) is assumed to be the alagator field Elec.}
  lappend usage {The expand flag is used to expand the terms into a full string for testing purposes.}
  set returns {nothing}
  set description {Dan Cumming's mobility model, sets a bunch of terms and returns nothing. Behavior is subject to change.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set T [__GetValue $args T "$name Error: No temperature (T) specified"]
  set expand [__Contains $args expand]
  set Accept [__GetValue $args Accept ]
  if {$Accept == ""} { set Accept Accept }
  set Donor [__GetValue $args Donor ]
  if {$Donor == ""} { set Donor Donor }
  set Elec [__GetValue $args Elec ]
  if {$Elec == ""} { set Elec Elec }
  set Hole [__GetValue $args Hole ]
  if {$Hole == ""} { set Hole Hole }
  if { $expand } { puts "Expansion not available yet"; exit; }
  
  set ND ($Donor)
  set NA ($Accept)

  #------------------------
  # 1. Lattice Scattering
  #------------------------
  
  set T_0 ($T)
  set T_Lattice ($T)
  
  #Phosphorus
  set mu_e_max 1414.0
  set mu_min_e 68.5
  set theta_e 2.285
  set N_eref 9.2e16
  set alpha_e 0.711
  
  #Boron
  set mu_h_max 470.5
  set mu_min_h 44.9
  set theta_h 2.247
  set N_href 2.23e17
  set alpha_h 0.719
  
  term add name=mu_eL eqn = "$mu_e_max*(($T_Lattice/$T_0)^(-$theta_e))"
  term add name=mu_hL eqn = "$mu_h_max*(($T_Lattice/$T_0)^(-$theta_h))"
  
  term add name=mu_eN eqn = "(($mu_e_max^2)/($mu_e_max-$mu_min_e))*(($T_Lattice/$T_0)^(3*$alpha_e-1.5))"
  term add name=mu_hN eqn = "(($mu_h_max^2)/($mu_h_max-$mu_min_h))*(($T_Lattice/$T_0)^(3*$alpha_h-1.5))"
  
  term add name=mu_ec eqn = "(($mu_e_max*$mu_min_e)/($mu_e_max-$mu_min_e))*(($T_Lattice/$T_0)^0.5)"
  term add name=mu_hc eqn = "(($mu_h_max*$mu_min_h)/($mu_h_max-$mu_min_h))*(($T_Lattice/$T_0)^0.5)"
  
  #-------------------------------
  # High-Concentration Clustering
  #-------------------------------
  
  set c_D 0.21
  set c_A 0.5
  set N_refD 7.2e20
  set N_refA 4.0e20
  
  term add name=ND_clus eqn = "$ND*(1+(1/($c_D+(($N_refD/$ND)^2))))"
  term add name=NA_clus eqn = "$NA*(1+(1/($c_A+(($N_refA/$NA)^2))))"
  
  term add name=N_esc eqn = "ND_clus + NA_clus + $Hole"
  term add name=N_hsc eqn = "ND_clus + NA_clus + $Elec"
  
  #---------------------
  # Screening Parameter
  #---------------------
  
  set me_div_m0 1.0
  set mh_div_m0 1.258
  set f_CW 2.459
  set f_BH 3.828
  
  term add name=Pe eqn = "(($f_CW/(3.97*10^13*(N_esc^(-2/3)))+$f_BH/(((1.36*10^20)/($Elec+$Hole))*$me_div_m0))^-1)*(($T_Lattice/$T_0)^2)"
  term add name=Ph eqn = "(($f_CW/(3.97*10^13*(N_hsc^(-2/3)))+$f_BH/(((1.36*10^20)/($Elec+$Hole))*$mh_div_m0))^-1)*(($T_Lattice/$T_0)^2)"
  
  #------------------------------
  # Minority Impurity Scattering
  #------------------------------
  
  set a_g 0.89233
  set b_g 0.41372
  set c_g 0.005978
  set alpha_g 0.28227
  set alphaprime_g 0.72169
  set beta_g 0.19778
  set gamma_g 1.80618
  set m0_div_me 1.0
  set m0_div_mh 0.794913
  
  term add name=G_Pe eqn = "1-$a_g/(($b_g+Pe*(($m0_div_me*($T_Lattice/$T_0))^$alpha_g))^$beta_g)+$c_g/((Pe*(($m0_div_me*($T_Lattice/$T_0))^$alphaprime_g))^$gamma_g)"
  term add name=G_Ph eqn = "1-$a_g/(($b_g+Ph*(($m0_div_mh*($T_Lattice/$T_0))^$alpha_g))^$beta_g)+$c_g/((Ph*(($m0_div_mh*($T_Lattice/$T_0))^$alphaprime_g))^$gamma_g)"
  
  #--------------------------
  # Electron-Hole Scattering
  #--------------------------
  
  set me_div_mh 0.794913
  set mh_div_me 1.258
  
  term add name=F_Pe eqn = "(0.7643*(Pe^0.6478)+2.2999+6.5502*$me_div_mh)/(Pe^0.6478+2.367-0.8552*$me_div_mh)"
  term add name=F_Ph eqn = "(0.7643*(Ph^0.6478)+2.2999+6.5502*$mh_div_me)/(Ph^0.6478+2.367-0.8552*$mh_div_me)"
  
  #--------------------------
  # Mobility Model Equations
  #--------------------------
  
  term add name=N_e_sceff eqn = "ND_clus+G_Pe*NA_clus+$Hole/F_Pe"
  term add name=N_h_sceff eqn = "NA_clus+G_Ph*ND_clus+$Elec/F_Ph"
  
  term add name=mu_e_DAeh eqn = "mu_eN*(N_esc/N_e_sceff)*(($N_eref/N_esc)^$alpha_e)+mu_ec*(($Elec+$Hole)/N_e_sceff)"
  term add name=mu_h_DAeh eqn = "mu_hN*(N_hsc/N_h_sceff)*(($N_href/N_hsc)^$alpha_h)+mu_hc*(($Elec+$Hole)/N_h_sceff)"
  
  term add name=Emob_Philips eqn = "(mu_e_DAeh*mu_eL)/(mu_e_DAeh+mu_eL)"
  term add name=Hmob_Philips eqn = "(mu_h_DAeh*mu_hL)/(mu_h_DAeh+mu_hL)"
  return "Emob_Philips Hmob_Philips"
}


}
