proc XBandEdgeShiftStrain args {
  set name [lindex [info level 0] 0]
  lappend usage "$name strain={StrainXX StrainYY StrainZZ StrainYZ StrainXZ StrainXY} miller=(001)<100> mat=Silicon"
  lappend usage "Strain vector is required to be in Voigt notation string list format."
  lappend usage "Miller indicies (miller) are in (hkl)<uvw> format and defaults to 001 wafers with 100 channels."
  lappend usage "Material (mat) is optional and defaults to silicon."
  set returns "list of 3 conduction band energy band level shifts in eV"
  set description { Uses deformation potentials to compute X valley shifts on the three principle axies }
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set strain [__GetValue $args strain]
  if { $strain == "" } {
    set strain {StrainXX StrainYY StrainZZ StrainYZ StrainXZ StrainXY}
  }
  set miller [__GetValue $args miller]
  if {$miller == ""} { set miller (001)<100> }
  set Material [__GetValue $args mat ]
  if {$Material == ""} { set Material Silicon }
  set Material Silicon

  if {[llength $strain] != 6} {
    puts "Bad strain vector length [llength $strain] in $name"
    exit
  }
  
  if { [ pdbIsAvailable $Material XValley Xiu  ] } { set Xiu  [ pdbGet $Material XValley Xiu  ] } else {
    puts "Xiu constant not specified for material $Material"; exit;
  }
  if { [ pdbIsAvailable $Material XValley Xid  ] } { set Xid  [ pdbGet $Material XValley Xid  ] } else {
    puts "Xid constant not specified for material $Material"; exit;
  }
  if { [ pdbIsAvailable $Material XValley Xiup  ] } { set Xiup  [ pdbGet $Material XValley Xiup  ] } else {
    set Xiup 0;
  }
  set DP [list "$Xiu+$Xid $Xid      $Xid      0.0   0.0   0.0" \
               "$Xid      $Xiu+$Xid $Xid      0.0   0.0   0.0" \
               "$Xid      $Xid      $Xiu+$Xid 0.0   0.0   0.0" \
               "0.0       0.0       0.0       $Xiup 0.0   0.0" \
               "0.0       0.0       0.0       0.0   $Xiup 0.0" \
               "0.0       0.0       0.0       0.0   0.0   $Xiup"]
  
  # lets build the conduction list based on strain expressions
  set Econdlist "1 2 3 4 5 6"
  for {set i 0} {$i<6} {incr i} {
    set tmp ""
    for {set j 0} {$j<6} {incr j} {
      set DP_val [expr [lindex $DP $i $j]]
      # if the DP value is 0 no need to waste parsing time
      if { abs($DP_val) < 1e-10 } { continue }
      if { $tmp == "" } {
        set tmp "($DP_val*[lindex $strain $j])"
      } else {
        set tmp "$tmp+($DP_val*[lindex $strain $j])"
      }
    }
    lset Econdlist $i $tmp
  }

  return $Econdlist
  
}

proc XBandEdgeShiftStress args {
  set name [lindex [info level 0] 0]
  lappend usage "$name stress={1 2 3 4 5 6} miller=(001)<100> mat=Silicon"
  lappend usage "Stress vector is required to be in Voigt notation string list format."
  lappend usage "Miller indicies (miller) are in (hkl)<uvw> format and defaults to 001 wafers with 100 channels."
  lappend usage "Material (mat) is optional and defaults to silicon."
  set returns "list of 3 conduction band energy band level shifts in eV"
  set description { Uses deformation potentials to compute X valley shifts on the three principle axies }
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set stress [__GetValue $args stress "$name Error: No stress vector provided"]
  set miller [__GetValue $args miller]
  if {$miller == ""} { set miller (001)<100> }
  set Material [__GetValue $args mat ]
  if {$Material == ""} { set Material Silicon }
  set Material Silicon

  if {[llength $stress] != 6} {
    puts "Bad stress vector in $name"
    exit
  }
  
  
  # rotate stress to principle axis
  set alpha [__approxFirstToSecondOrderRotation [ __MillerToRotation $miller ]]
  set stress [ __MultAx $alpha $stress ]
  # get compliance tensor on principle axies
  set S [ __Compliance (001)<100> $Material ]
  
  if { [ pdbIsAvailable $Material XValley Xiu  ] } { set Xiu  [ pdbGet $Material XValley Xiu  ] } else {
    puts "Xiu constant not specified for material $Material"; exit;
  }
  if { [ pdbIsAvailable $Material XValley Xid  ] } { set Xid  [ pdbGet $Material XValley Xid  ] } else {
    puts "Xid constant not specified for material $Material"; exit;
  }
  if { [ pdbIsAvailable $Material XValley Xiup  ] } { set Xiup  [ pdbGet $Material XValley Xiup  ] } else {
    set Xiup 0;
  }
  set DP [list "$Xiu+$Xid $Xid      $Xid      0.0   0.0   0.0" \
               "$Xid      $Xiu+$Xid $Xid      0.0   0.0   0.0" \
               "$Xid      $Xid      $Xiu+$Xid 0.0   0.0   0.0" \
               "0.0       0.0       0.0       $Xiup 0.0   0.0" \
               "0.0       0.0       0.0       0.0   $Xiup 0.0" \
               "0.0       0.0       0.0       0.0   0.0   $Xiup"]
  
  for {set i 0} {$i<6} {incr i} {
    for {set j 0} {$j<6} {incr j} {
      lset DP $i $j [expr [lindex $DP $i $j]]
    }
  }
  
  set Econdlist [__MultAx $DP [__MultAx $S $stress ]]
  
  return $Econdlist
  
}

proc DTau args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T= Elec= Elec0=Elec0"
  set returns "Mobility change"
  set description { Uses deformation potentials to compute X valley shifts on the three principle axies }
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set Elec [__GetValue $args Elec "$name Error: No Elec concentration provided"]
  set T [__GetValue $args T "$name Error: No temperature (T) concentration provided"]
  set Elec0 [__GetValue $args Elec0]
  if {$Elec0 == ""} { set Elec0 Elec0 }

  set q 1.602e-19
  set k 1.38066e-23
  return "(($T)*[expr {$k/$q}]*(($Elec / $Elec0) - 1.0))"
}

proc __getXmobs { StressVector6 Miller Elecs Material {raw 0} } {
  
  set alpha [__approxFirstToSecondOrderRotation [ __MillerToRotation $Miller ]]
  set S [ __Compliance (001)<100> $Material ]
  set Strain [__MultAx $S [__MultAx $alpha $StressVector6 ]]
  
  if { [ pdbIsAvailable $Material XTotalMass ] } { set XTotalMass [ pdbGet $Material XTotalMass ] } else {
    puts "X Band total electron mass (XTotalMass) not specifited for material $Material"; exit;
  }
  if { [ pdbIsAvailable $Material XTransverseMass  ] } { set XTransverseMass [ pdbGet $Material XTransverseMass ] } else {
    puts "X Band Transverse electron mass (XTransverseMass) not specifited for material $Material"; exit;
  }
  if { [ pdbIsAvailable $Material XLongitudinalMass  ] } { set XLongitudinalMass [ pdbGet $Material XLongitudinalMass ] } else {
    puts "X Band Longitudinal electron mass (XLongitudinalMass) not specifited for material $Material"; exit;
  }
  
  set Emob_long [ expr $XTotalMass/$XLongitudinalMass ]
  set Emob_tran [ expr $XTotalMass/$XTransverseMass ]
  
  if { [ pdbIsAvailable $Material f  ] } { set f [ pdbGet $Material f ] } else {
    puts "Scattering shift constant not specified for material $Material"; exit;
  }
  
  set D1   "(1.0+$f*DTau100)"
  set D2   "(1.0+$f*DTau010)"
  set D3   "(1.0+$f*DTau001)"
  
  if { [ pdbIsAvailable $Material alpha  ] } { set splitConst [ pdbGet $Material alpha ] } else {
    puts "Splitting constant alpha not specified for material $Material"; exit;
  }
  #set splitConst 86.8 ;# usually denoted as alpha
  set splitYZ [ expr $splitConst/2.*[lindex $Strain 3] ]
  set splitXZ [ expr $splitConst/2.*[lindex $Strain 4] ]
  set splitXY [ expr $splitConst/2.*[lindex $Strain 5] ]
  set mobVecX "$Emob_long [expr $Emob_tran*(1-$splitYZ)] [expr $Emob_tran*(1+$splitYZ)]"
  set mobVecY "[expr $Emob_tran*(1-$splitXZ)] $Emob_long [expr $Emob_tran*(1+$splitXZ)]"
  set mobVecZ "[expr $Emob_tran*(1-$splitXY)] [expr $Emob_tran*(1+$splitXY)] $Emob_long"
  if { ! $raw } {
    set mobVecX "([lindex $mobVecX 0]*$D1) ([lindex $mobVecX 1]*$D1) ([lindex $mobVecX 2]*$D1) 0.0 0.0 0.0"
    set mobVecY "([lindex $mobVecY 0]*$D2) ([lindex $mobVecY 1]*$D2) ([lindex $mobVecY 2]*$D2) 0.0 0.0 0.0"
    set mobVecZ "([lindex $mobVecZ 0]*$D3) ([lindex $mobVecZ 1]*$D3) ([lindex $mobVecZ 2]*$D3) 0.0 0.0 0.0"
  }
  return "\"$mobVecX\"\
          \"$mobVecY\"\
          \"$mobVecZ\""
}

proc __getXmobsStrain { Elecs Material {raw 0} } {
  
  set Strain {StrainXX StrainYY StrainZZ StrainYZ StrainXZ StrainXY}
  if { [ pdbIsAvailable $Material XTotalMass ] } { set XTotalMass [ pdbGet $Material XTotalMass ] } else {
    puts "X Band total electron mass (XTotalMass) not specifited for material $Material"; exit;
  }
  if { [ pdbIsAvailable $Material XTransverseMass  ] } { set XTransverseMass [ pdbGet $Material XTransverseMass ] } else {
    puts "X Band Transverse electron mass (XTransverseMass) not specifited for material $Material"; exit;
  }
  if { [ pdbIsAvailable $Material XLongitudinalMass  ] } { set XLongitudinalMass [ pdbGet $Material XLongitudinalMass ] } else {
    puts "X Band Longitudinal electron mass (XLongitudinalMass) not specifited for material $Material"; exit;
  }
  
  set Emob_long [ expr $XTotalMass/$XLongitudinalMass ]
  set Emob_tran [ expr $XTotalMass/$XTransverseMass ]
  
  if { [ pdbIsAvailable $Material f  ] } { set f [ pdbGet $Material f ] } else {
    puts "Scattering shift constant not specified for material $Material"; exit;
  }
  
  set D1 "(1.0+$f*DTau100)"
  set D2 "(1.0+$f*DTau010)"
  set D3 "(1.0+$f*DTau001)"
  
  if { [ pdbIsAvailable $Material alpha  ] } { set splitConst [ pdbGet $Material alpha ] } else {
    puts "Splitting constant alpha not specified for material $Material"; exit;
  }
  #set splitConst 86.8 ;# usually denoted as alpha
  set splitYZ ($splitConst/2.*[lindex $Strain 3])
  set splitXZ ($splitConst/2.*[lindex $Strain 4])
  set splitXY ($splitConst/2.*[lindex $Strain 5])
  set mobVecX "$Emob_long ($Emob_tran*(1-$splitYZ)) ($Emob_tran*(1+$splitYZ))"
  set mobVecY "($Emob_tran*(1-$splitXZ)) $Emob_long ($Emob_tran*(1+$splitXZ))"
  set mobVecZ "($Emob_tran*(1-$splitXY)) ($Emob_tran*(1+$splitXY)) $Emob_long"
  if { ! $raw } {
    set mobVecX "([lindex $mobVecX 0]*$D1) ([lindex $mobVecX 1]*$D1) ([lindex $mobVecX 2]*$D1) 0.0 0.0 0.0"
    set mobVecY "([lindex $mobVecY 0]*$D2) ([lindex $mobVecY 1]*$D2) ([lindex $mobVecY 2]*$D2) 0.0 0.0 0.0"
    set mobVecZ "([lindex $mobVecZ 0]*$D3) ([lindex $mobVecZ 1]*$D3) ([lindex $mobVecZ 2]*$D3) 0.0 0.0 0.0"
  }
  return "\"$mobVecX\"\
          \"$mobVecY\"\
          \"$mobVecZ\""
}

